@@ -6,8 +6,9 @@
 #include "artistbrowser.h"
 #include "album.h"
 #include "session.h"
+#include "track.h"
 
-static PyObject *
+PyObject *
 ArtistBrowser_FromSpotify(sp_artistbrowse * browse)
 {
     ArtistBrowser *b = (ArtistBrowser *)ArtistBrowserType.tp_alloc(&ArtistBrowserType, 0);
@@ -19,11 +20,12 @@ ArtistBrowser_FromSpotify(sp_artistbrowse * browse)
 }
 
 void
-ArtistBrowser_browse_complete(sp_artistbrowse * browse, Callback * st)
+ArtistBrowser_browse_complete(sp_artistbrowse *browse, Callback *st)
 {
 #ifdef DEBUG
-    fprintf(stderr, "Artist browse complete\n");
+    fprintf(stderr, "[DEBUG]-artbrw- browse complete (%p, %p)\n", browse, st);
 #endif
+    if (!st) return;
     PyGILState_STATE gstate = PyGILState_Ensure();
     PyObject *browser = ArtistBrowser_FromSpotify(browse);
 
@@ -31,6 +33,7 @@ ArtistBrowser_browse_complete(sp_artistbrowse * browse, Callback * st)
                                                  st->userdata, NULL);
     if (!res)
         PyErr_WriteUnraisable(st->callback);
+    delete_trampoline(st);
     Py_DECREF(browser);
     Py_XDECREF(res);
     PyGILState_Release(gstate);
@@ -39,44 +42,51 @@ ArtistBrowser_browse_complete(sp_artistbrowse * browse, Callback * st)
 static PyObject *
 ArtistBrowser_new(PyTypeObject * type, PyObject *args, PyObject *kwds)
 {
-    PyObject *session, *artist, *callback, *userdata = NULL;
+    PyObject *artist, *callback = NULL, *userdata = NULL;
+    char *str_type = NULL;
+    sp_artistbrowse_type abtype = SP_ARTISTBROWSE_FULL;
+    Callback *cb = NULL;
+    ArtistBrowser *self;
     static char *kwlist[] =
-        { "session", "artist", "callback", "userdata", NULL };
+        { "artist", "type", "callback", "userdata", NULL };
+
     if (!PyArg_ParseTupleAndKeywords
-        (args, kwds, "O!O!O|O", kwlist, &SessionType, &session, &ArtistType,
-         &artist, &callback, &userdata))
+        (args, kwds, "O!|sOO", kwlist, &ArtistType, &artist, &str_type,
+         &callback, &userdata))
         return NULL;
-
-    ArtistBrowser *self = (ArtistBrowser *) type->tp_alloc(type, 0);
-
-    self->_callback.callback = callback;
-    self->_callback.userdata = userdata;
-    Py_XINCREF(callback);
-    Py_XINCREF(userdata);
-
-    Py_BEGIN_ALLOW_THREADS;
+    if (str_type) {
+        if (strcmp(str_type, "full") == 0) {} // default
+        else if (strcmp(str_type, "no_tracks") == 0) {
+            abtype = SP_ARTISTBROWSE_NO_TRACKS;
+        }
+        else if (strcmp(str_type, "no_albums") == 0) {
+            abtype = SP_ARTISTBROWSE_NO_ALBUMS;
+        }
+        else {
+            PyErr_SetString(PyExc_ValueError, "Unknown artist browser type.");
+            return NULL;
+        }
+    }
+    self = (ArtistBrowser *) type->tp_alloc(type, 0);
+    if (callback) {
+        if (!userdata)
+            userdata = Py_None;
+        cb = create_trampoline(callback, NULL, userdata);
+    }
     self->_browser =
-        sp_artistbrowse_create(((Session *) session)->_session,
+        sp_artistbrowse_create(g_session,
                                ((Artist *) artist)->_artist,
-                               SP_ARTISTBROWSE_FULL,
+                               abtype,
                                (artistbrowse_complete_cb *)
                                ArtistBrowser_browse_complete,
-                               (void *)&self->_callback);
-    Py_END_ALLOW_THREADS;
+                               cb);
     return (PyObject *)self;
 }
 
 static void
 ArtistBrowser_dealloc(PyObject *arg)
 {
-#ifdef DEBUG
-    fprintf(stderr, "Deallocating artist browser");
-#endif
-
     ArtistBrowser *self = (ArtistBrowser *) arg;
-
-    Py_XDECREF(self->_callback.callback);
-    Py_XDECREF(self->_callback.userdata);
     sp_artistbrowse_release(self->_browser);
     ArtistBrowserType.tp_free(self);
 }
@@ -87,21 +97,85 @@ ArtistBrowser_is_loaded(ArtistBrowser * self)
     return Py_BuildValue("i", sp_artistbrowse_is_loaded(self->_browser));
 }
 
+static PyObject *
+ArtistBrowser_albums(ArtistBrowser * self)
+{
+    sp_album *album;
+    int count = sp_artistbrowse_num_albums(self->_browser);
+    PyObject *l = PyList_New(count);
+    int i;
+
+    for (i = 0; i < count; ++i) {
+        album = sp_artistbrowse_album(self->_browser, i);
+        PyObject *a = Album_FromSpotify(album);
+
+        PyList_SetItem(l, i, a);
+    }
+    return l;
+}
+
+static PyObject *
+ArtistBrowser_similar_artists(ArtistBrowser * self)
+{
+    sp_artist *artist;
+    int count = sp_artistbrowse_num_similar_artists(self->_browser);
+    PyObject *l = PyList_New(count);
+    int i;
+
+    for (i = 0; i < count; ++i) {
+        artist = sp_artistbrowse_similar_artist(self->_browser, i);
+        PyObject *a = Artist_FromSpotify(artist);
+
+        PyList_SetItem(l, i, a);
+    }
+    return l;
+}
+
+static PyObject *
+ArtistBrowser_tracks(ArtistBrowser * self)
+{
+    int count = sp_artistbrowse_num_tracks(self->_browser);
+    PyObject *l = PyList_New(count);
+    int i;
+
+    for (i = 0; i < count; ++i) {
+        PyObject *a = Track_FromSpotify(sp_artistbrowse_track(self->_browser, i));
+
+        PyList_SetItem(l, i, a);
+    }
+    return l;
+}
+
+static PyObject *
+ArtistBrowser_tophit_tracks(ArtistBrowser * self)
+{
+    int count = sp_artistbrowse_num_tophit_tracks(self->_browser);
+    PyObject *l = PyList_New(count);
+    int i;
+
+    for (i = 0; i < count; ++i) {
+        PyObject *a = Track_FromSpotify(sp_artistbrowse_tophit_track(self->_browser, i));
+
+        PyList_SetItem(l, i, a);
+    }
+    return l;
+}
+
 Py_ssize_t
 ArtistBrowser_sq_length(ArtistBrowser * self)
 {
-    return sp_artistbrowse_num_albums(self->_browser);
+    return sp_artistbrowse_num_tracks(self->_browser);
 }
 
 PyObject *
 ArtistBrowser_sq_item(ArtistBrowser * self, Py_ssize_t index)
 {
-    if (index >= sp_artistbrowse_num_albums(self->_browser)) {
+    if (index >= sp_artistbrowse_num_tracks(self->_browser)) {
         PyErr_SetString(PyExc_IndexError, "");
         return NULL;
     }
-    sp_album *album = sp_artistbrowse_album(self->_browser, (int)index);
-    PyObject *wrapper = Album_FromSpotify(album);
+    sp_track *track = sp_artistbrowse_track(self->_browser, (int)index);
+    PyObject *wrapper = Track_FromSpotify(track);
 
     return wrapper;
 }
@@ -122,6 +196,22 @@ static PyMethodDef ArtistBrowser_methods[] = {
      (PyCFunction)ArtistBrowser_is_loaded,
      METH_NOARGS,
      "True if this artist browser has finished loading"},
+    {"albums",
+     (PyCFunction)ArtistBrowser_albums,
+     METH_NOARGS,
+     "Return a list of all the albums found while browsing."},
+    {"similar_artists",
+     (PyCFunction)ArtistBrowser_similar_artists,
+     METH_NOARGS,
+     "Return a list of all the artists found while browsing."},
+    {"tracks",
+     (PyCFunction)ArtistBrowser_tracks,
+     METH_NOARGS,
+     "Return a list of all the tracks found while browsing."},
+    {"tophit_tracks",
+     (PyCFunction)ArtistBrowser_tophit_tracks,
+     METH_NOARGS,
+     "Return a list of all the top tracks found while browsing."},
     {NULL}
 };
 
